{$apptype CONSOLE}
{.$define VERBOSE}
{$MAXSTACKSIZE 160000000}
{$R+,Q+,O-}
uses Math, SysUtils;

const
    IN_FILE = 'thruput.in';
    OUT_FILE = 'thruput.out';

    MAX_N = 100000;

type
    TEdge = record
        head, tail : integer;
        key : integer;
        nextout : integer;
        stripe : integer;
        pos : integer;
    end;

    TVertex = record
        parentEdge : integer;
        heavyEdge : integer;
        firstEdge : integer;
        enterItem : integer;
        rank : integer;
        stripe : integer;
        pos : integer;
    end;

    TItem = record
        key : integer;
        data : integer;
    end;

    TItems = array[1..MAX_N * 2] of TItem;

    TSegmentTree = record
        n, k : integer;
        items : array of TItem;
    end;

    TStripe = record
        tree : TSegmentTree;
        head : integer;
    end;

var
    n : integer;
    e : array[1..MAX_N * 2] of TEdge;
    v : array[1..MAX_N] of TVertex;

    items : TItems;
    nItems : integer;

    time : integer;
    lcaTree : TSegmentTree;

    nStripes : integer;
    stripes : array[1..MAX_N] of TStripe;

{$ifdef VERBOSE}
    totalLength : integer;
{$endif}

procedure ResetItems;
begin
    nItems := 0;
end;

function AppendItem(data, key : integer) : integer;
begin
    inc(nItems);
    items[nItems].data := data;
    items[nItems].key := key;
    result := nItems;
end;

procedure EvalSegmentTreeItem(const tree : TSegmentTree; i : integer);
begin
    if (i * 2 > tree.k + tree.n - 1) or (tree.items[i*2].data = 0) then begin
        tree.items[i].data := 0;
        exit;
    end;
    if (tree.items[2*i + 1].data = 0) or (tree.items[i*2].key < tree.items[i*2+1].key) then
        tree.items[i] := tree.items[i*2]
    else
        tree.items[i] := tree.items[i*2+1];
end;

function Log2up(n : integer) : integer;
begin
    result := 1;
    while result < n do
        result := result * 2;
end;

function CreateSegmentTree(const items : TItems; n : integer) : TSegmentTree;
var
    i : integer;

begin
    result.n := n;
    result.k := Log2up(n);
    setlength(result.items, 2 * result.k);
    for i := 1 to n do
        result.items[i + result.k - 1] := items[i];
    for i := result.k - 1 downto 1 do
        EvalSegmentTreeItem(result, i);
end;

function QuerySegmentTree(const tree : TSegmentTree; lo, hi : integer) : TItem;
var
    best : integer;

    procedure Walk(i, left, right, lo, hi : integer);
    var
        mid : integer;

    begin
        if (left = lo) and (right = hi) then begin
            if (best = 0) or (tree.items[best].key > tree.items[i].key) then
                best := i;
            exit;
        end;

        mid := (left + right) div 2;
        if lo <= mid then
            Walk(2 * i, left, mid, lo, min(mid, hi));
        if hi >= mid + 1 then
            Walk(2 * i + 1, mid + 1, right, max(mid + 1, lo), hi);
    end;

begin
{$ifdef VERBOSE}
    inc(totalLength, hi - lo + 1);
{$endif}
    best := 0;
    Walk(1, 1, tree.k, lo, hi);
    result := tree.items[best];
end;

procedure UpdateSegmentTree(const tree : TSegmentTree; pos : integer; key : integer);
var
    i : integer;

begin
    i := pos + tree.k - 1;
    tree.items[i].key := key;
    while i <> 1 do begin
        i := i div 2;
        EvalSegmentTreeItem(tree, i);
    end;
end;

procedure ReadData;
var
    i : integer;
    m : integer;
    x, y : integer;
    key : integer;

    procedure InsertEdge(tail, head, key : integer);
    begin
        inc(m);
        e[m].head := head;
        e[m].tail := tail;
        e[m].key := key;
        e[m].nextout := v[tail].firstEdge;
        v[tail].firstEdge := m;
    end;

begin
    readln(n);
    if (n < 1) or (n > MAX_N) then
        runerror(239);
    fillchar(v, sizeof(v), 0);
    fillchar(e, sizeof(e), 0);
    m := 0;
    for i := 1 to n - 1 do begin
        readln(x, y, key);
        if (x < 1) or (x > n) or (y < 1) or (y > n) then
            runerror(239);
        if (key < 1) or (key > 1000000) then
            runerror(239);
        InsertEdge(x, y, key);
        InsertEdge(y, x, key);
    end;
end;

procedure PrepareLca;

    procedure EulerDFS(x, parent, parentEdge : integer);
    var
        xtime, i, he : integer;

    begin
        xtime := time;
        inc(time);

        v[x].enterItem := AppendItem(x, xtime);
        v[x].parentEdge := parentEdge;
        v[x].rank := 1;
        he := 0;
        i := v[x].firstEdge;
        while i <> 0 do begin
            if e[i].head <> parent then begin
                EulerDFS(e[i].head, x, i);
                if (he = 0) or (v[e[he].head].rank < v[e[i].head].rank) then
                    he := i;
                inc(v[x].rank, v[e[i].head].rank);
                AppendItem(x, xtime);
            end;
            i := e[i].nextout;
        end;
        v[x].heavyEdge := he;
    end;

begin
    time := 1;
    ResetItems;
    EulerDFS(1, 0, 0);
    assert(nItems = 2 * n - 1);
    lcaTree := CreateSegmentTree(items, nItems);
end;

function Lca(a, b : integer) : integer;
var
    lo, hi : integer;

begin
    lo := min(v[a].enterItem, v[b].enterItem);
    hi := max(v[a].enterItem, v[b].enterItem);
    result := QuerySegmentTree(lcaTree, lo, hi).data;
end;

procedure PrepareStripes;

    procedure MakeStripe(x, parent : integer);
    var
        i : integer;

    begin
        inc(nStripes);
        ResetItems;
        i := x;
        repeat
            v[i].stripe := nStripes;
            v[i].pos := nItems + 1;
            if v[i].heavyEdge = 0 then
                break;
            AppendItem(v[i].heavyEdge, -e[v[i].heavyEdge].key);
            e[v[i].heavyEdge].stripe := nStripes;
            e[v[i].heavyEdge].pos := nItems;
            i := e[v[i].heavyEdge].head;
        until false;

        stripes[nStripes].head := x;
        stripes[nStripes].tree := CreateSegmentTree(items, nItems);
    end;

    procedure DFS(x, parent : integer);
    var
        i : integer;

    begin
        if v[x].stripe = 0 then
            MakeStripe(x, parent);

        i := v[x].firstEdge;
        while i <> 0 do begin
            if e[i].head <> parent then
                DFS(e[i].head, x);
            i := e[i].nextout;
        end;
    end;

begin
    nStripes := 0;
    DFS(1, 0);
end;

function QueryMaxEdgeSimple(a, b : integer) : integer;
var
    minEdge : integer;

    procedure UpdateMin(i : integer);
    begin
        if (minEdge = 0) or (e[minEdge].key < e[i].key) then
            minEdge := i;
    end;

begin
    minEdge := 0;
    while v[a].stripe <> v[b].stripe do begin
        if v[a].pos > 1 then
            UpdateMin(QuerySegmentTree(stripes[v[a].stripe].tree, 1, v[a].pos - 1).data);
        a := stripes[v[a].stripe].head;
        UpdateMin(v[a].parentEdge);
        a := e[v[a].parentEdge].tail;
    end;

    if a <> b then begin
        UpdateMin(QuerySegmentTree(stripes[v[a].stripe].tree, v[b].pos, v[a].pos - 1).data);
    end;

    result := minEdge;
end;

function QueryMaxEdge(a, b : integer) : integer;
var
    c, ac, bc : integer;

begin
    c := lca(a, b);
    ac := QueryMaxEdgeSimple(a, c);
    bc := QueryMaxEdgeSimple(b, c);
    if (bc = 0) or ((ac <> 0) and (e[ac].key > e[bc].key)) then
        result := ac
    else
        result := bc;
end;

procedure UpdateEdge(i : integer; key : integer);
begin
    e[i].key := key;
    if e[i].stripe = 0 then
        exit;
    UpdateSegmentTree(stripes[e[i].stripe].tree, e[i].pos, -key);
end;

function strtok(var s : string) : string;
var
    i : integer;

begin
    s := Trim(s);
    i := pos(' ', s);
    if i = 0 then begin
        result := s;
        s := '';
    end else begin
        result := copy(s, 1, i - 1);
        delete(s, 1, i);
    end;
end;

var
    a, b : integer;
    line, cmd : string;

begin
    assign(input, IN_FILE);
    reset(input);
    ReadData;

    PrepareLca;
    PrepareStripes;

    assign(output, OUT_FILE);
    rewrite(output);

    repeat
        readln(line);
        cmd := strtok(line);
        if cmd = 'DONE' then
            break
        else if cmd = 'QUERY' then begin
            a := strtoint(strtok(line));
            b := strtoint(strtok(line));
            if (a < 1) or (a > n) or (b < 1) or (b > n) or (a = b) then
                runerror(239);
            writeln(e[QueryMaxEdge(a, b)].key);
        end else if cmd = 'CHANGE' then begin
            a := strtoint(strtok(line));
            b := strtoint(strtok(line));
            if (a < 1) or (a > n) then
                runerror(239);
            if (b < 1) or (b > 1000000) then
                runerror(239);
            UpdateEdge(2 * a, b);
            UpdateEdge(2 * a - 1, b);
        end else
            runerror(239);
    until false;

{$ifdef VERBOSE}
    writeln('total length = ', totalLength);
{$endif}
    close(output);
end.

